/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.bpmn2designer;

import javax.xml.namespace.QName;
import org.ow2.orchestra.designer.bpmn.model.PoolModel;
import org.ow2.orchestra.designer.bpmn.model.operations.InterfaceModel;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.Import;
import org.ow2.orchestra.jaxb.bpmn.Participant;

/**
 * Transform {@link org.ow2.orchestra.jaxb.bpmn.Participant}s into {@link org.ow2.orchestra.designer.bpmn.model.PoolModel}s and vice versa
 *
 * @author Guillaume Porcher
 */
public final class PoolTransformer {

  /**
   * Utility classes should not have public or default constructors.
   */
  private PoolTransformer() {

  }

  /**
   * @param participant
   * @param context
   * @return
   * @throws org.ow2.orchestra.designer.models.exceptions.TransformationException
   *
   */
  public static void bpmnToModel(final Participant participant, final Bpmn2DesignerContext context) throws TransformationException {
    final PoolModel poolModel = new PoolModel();
    poolModel.setName(participant.getName());
    poolModel.setProcessModel(context.getProcessModel());
    context.getProcessModel().getElements().add(poolModel);

    if (participant.getId() == null || "".equals(participant.getId())) {
      // If there is no id, we will not be able to find out a diagram element
      // for this gateway.
      // We will not be able to reference this gateway from the designer, so this is an error
      throw new TransformationException("BPMN conversion error: A participant has no id.");
    }
    poolModel.setId(participant.getId());

    // Set interfaces
    Bpmn2DesignerTransformer.generatePoolModelInterfaces(context, poolModel, participant);

    for (InterfaceModel poolInterface : poolModel.getInterfaces().values()) {
      poolModel.setWsdlNamespace(QName.valueOf(poolInterface.getPortTypeQName()).getNamespaceURI());
    }
    for (Import bpmnImport : context.getDefinitions().getImports()) {
      if (bpmnImport.getImportType().equals("http://schemas.xmlsoap.org/wsdl/")
          && bpmnImport.getNamespace().equals(poolModel.getWsdlNamespace())) {
        poolModel.setWsdlPath(bpmnImport.getLocation());
      }
    }

    // Now retrieve positions
    ShapeTransformer.bpmnToModel(participant.getId(), poolModel, context);
  }

}
